import java.util.ArrayList;
import java.util.Arrays;
import java.util.Random;


public class DocumentsInOut {
	private String catName;
	private int catID;
	private int inCount;
	private int outCount;
	private int[] docsIn;
	private int[] docsOut;
	private int[] valIn;
	private int[] valOut;
	private int[] testIn;
	private int[] testOut;
	
	public DocumentsInOut()
	{
		
	}
	
	public DocumentsInOut(int[] in, int[] out)
	{
		this.fill(in, out);
	}
	
	public void fill(int[] in, int[] out)
	{
		
		docsIn = in.clone();
		docsOut = out.clone();
		
		
		inCount = docsIn.length;
		outCount = docsOut.length;
	}
	
	public void fillIn(int in[])
	{
		System.arraycopy(in, 0, docsIn, 0, in.length);
		inCount = docsIn.length;
	}
	
	public void fillOut(int[] out)
	{
		System.arraycopy(out, 0, docsOut, 0, out.length);
		outCount = docsOut.length;
	}
	
	public void fillVal(ArrayList<Integer> inDocs, ArrayList<Integer> outDocs)
	{
		valIn = new int[inDocs.size()];
		valOut = new int[outDocs.size()];
		for(int k = 0; k < inDocs.size(); k++) valIn[k] = inDocs.get(k).intValue(); 
		for(int k = 0; k < outDocs.size(); k++) valOut[k] = outDocs.get(k).intValue();
	}
	
	public void fillTest(ArrayList<Integer> inDocs, ArrayList<Integer> outDocs)
	{
		testIn = new int[inDocs.size()];
		testOut = new int[outDocs.size()];
		for(int k = 0; k < inDocs.size(); k++) testIn[k] = inDocs.get(k).intValue(); 
		for(int k = 0; k < outDocs.size(); k++) testOut[k] = outDocs.get(k).intValue();
	}
	
	public int[] getDocList(int number, double percent)
	{
		if(percent > 0.99999999 || percent < 0.00000001)
		{
			System.err.printf("Percentage must be between 0 and 1\n");
			System.exit(-1);
		}
		
		double curPer = (double)inCount/(double)(inCount + outCount);
		double usePercent = 0.0;
		
		if(curPer > percent && curPer < (1.0 - percent))
			usePercent = curPer;
		else if(curPer <= percent)
			usePercent = percent;
		else if(curPer > (1.0 - percent))
			usePercent = 1.0 - percent;
		
		System.out.printf("natural percentage: %g - used: %g\n", curPer, usePercent);
		// do we have enough documents in and out of the cat?
		double numReqIn = Math.ceil(usePercent*(double)number);
		double numReqOut = (double)number - numReqIn;
		
		int useInCount = (int)numReqIn;
		int useOutCount = (int)numReqOut;
		int newNumber = useInCount + useOutCount;
		
		System.out.printf("ri: %g\tro: %g\t ui: %d uo: %d\t in: %d\t out: %d\n",
				numReqIn, numReqOut, useInCount, useOutCount, inCount, outCount);
		
		if(numReqIn > inCount || numReqOut > outCount)
		{
			
			useInCount = inCount;
			newNumber = (int)((double)inCount/usePercent);
			useOutCount = newNumber - useInCount;
			// this should be allowed because the percentages checks the numbers
		}
		
		int[] docList = new int[newNumber];
		// copy and randomly sort the in and out array
		//int [] tmpDocsIn = new int[inCount];
		//System.arraycopy(docsIn, 0, tmpDocsIn, 0, inCount);
		int [] tmpDocsIn = docsIn.clone();
		//int [] tmpDocsOut = new int[outCount];
		//System.arraycopy(docsOut, 0, tmpDocsOut, 0, outCount);
		int [] tmpDocsOut = docsOut.clone();
			
		System.out.printf("In: %d - Out: %d\n", tmpDocsIn.length, tmpDocsOut.length);
		
//		Random generator = new Random(  );
//		for (int i = inCount - 1; i > 0; i--)
//		{
//			//choose a random position between i and n-1
//			int which = generator.nextInt(i);
//			//swap permutation[i] and permutation[which]
//			int tmp = tmpDocsIn[i];
//			tmpDocsIn[i] = tmpDocsIn[which];
//			tmpDocsIn[which] = tmp;
//		}
//		System.out.println("two");
//
//		generator = new Random(  );
//		for (int i = outCount - 1; i > 0; i--)
//		{
//			//choose a random position between i and n-1
//			int which = generator.nextInt(i);
//			//swap permutation[i] and permutation[which]
//			int tmp = tmpDocsOut[i];
//			tmpDocsOut[i] = tmpDocsOut[which];
//			tmpDocsOut[which] = tmp;
//		}
//		System.out.println("three");

		for(int i = 0; i < useInCount; i++)
			docList[i] = tmpDocsIn[i];
		
		for(int i = useInCount; i < useInCount + useOutCount; i++)
			docList[i] = tmpDocsOut[i - useInCount];
		
		//Arrays.sort(docList);
		
		return docList;
	}
	
	public int [] countInOut()
	{
		int [] counts = new int[2];
		counts[0] = inCount;
		counts[1] = outCount;
		return counts;
	}
	
	public int [] getDocsIn()
	{
		return docsIn;
	}
	
	public int [] getDocsOut()
	{
		return docsOut;
	}
	
	public int[] getValIn()
	{
		return valIn;
	}
	
	public int[] getValOut()
	{
		return valOut;
	}
	
	public int[] getTestIn()
	{
		return testIn;
	}
	public int[] getTestOut()
	{
		return testOut;
	}
	
	public String getFullUsage()
	{
		StringBuffer use = new StringBuffer();
		use.append(Arrays.toString(docsIn));
		use.append("\n");
		use.append(Arrays.toString(docsOut));
		use.append("\n");
		use.append(Arrays.toString(valIn));
		use.append("\n");
		use.append(Arrays.toString(valOut));
		use.append("\n");
		use.append(Arrays.toString(testIn));
		use.append("\n");
		use.append(Arrays.toString(testOut));
		use.append("\n");
		
		return use.toString();
	}
	
	public void loadDocsIn(String diString)
	{
		diString = diString.replaceAll("\\s+|\\[|\\]", "");
		//System.out.println(diString);
		String[] d = diString.split(",");
		docsIn = new int[d.length];
		for(int i = 0; i < d.length; i++)
			docsIn[i] = Integer.valueOf(d[i]);
		
		inCount = docsIn.length;
		
	}
	public void loadDocsOut(String doString)
	{
		doString = doString.replaceAll("\\s+|\\[|\\]", "");
		//System.out.println(doString);
		String[] d = doString.split(",");
		docsOut = new int[d.length];
		for(int i = 0; i < d.length; i++)
			docsOut[i] = Integer.valueOf(d[i]);
		
		outCount = docsOut.length;
		
	}
	
	public void loadFullUsage(String fullString)
	{
		// split the string into lines
		String[] lineSplit = fullString.split("\n");
		
		String d[] = lineSplit[0].replaceAll("\\s+|\\[|\\]", "").split(",");
		docsIn = new int[d.length];
		for(int i = 0; i < d.length; i++)
			docsIn[i] = Integer.valueOf(d[i]);
		inCount = docsIn.length;
		
		d = lineSplit[1].replaceAll("\\s+|\\[|\\]", "").split(",");
		docsOut = new int[d.length];
		for(int i = 0; i < d.length; i++)
			docsOut[i] = Integer.valueOf(d[i]);
		outCount = docsOut.length;
	
		d = lineSplit[2].replaceAll("\\s+|\\[|\\]", "").split(",");
		valIn = new int[d.length];
		for(int i = 0; i < d.length; i++)
			valIn[i] = Integer.valueOf(d[i]);
		
		d = lineSplit[3].replaceAll("\\s+|\\[|\\]", "").split(",");
		valOut = new int[d.length];
		for(int i = 0; i < d.length; i++)
			valOut[i] = Integer.valueOf(d[i]);
		
		d = lineSplit[4].replaceAll("\\s+|\\[|\\]", "").split(",");
		testIn = new int[d.length];
		for(int i = 0; i < d.length; i++)
			testIn[i] = Integer.valueOf(d[i]);
		
		d = lineSplit[5].replaceAll("\\s+|\\[|\\]", "").split(",");
		testOut = new int[d.length];
		for(int i = 0; i < d.length; i++)
			testOut[i] = Integer.valueOf(d[i]);
	}
}
