import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;


public class TermStats {

	private int catID;
	private String catName;
	private int[] termID;
	private double[] totalTermOccur;
	private double[] termOccurInClass;
	private double[] termOccurOutClass;
	private double[] termNotOccurInClass;
	private double[] termNotOccurOutClass;
	private double[] freqTermTotal;
	private double[] freqTermInClass;
	private double[] freqTermOutClass;
	private double[] mult;
	private double[] norm;
	private double[] IG;

	private double totalNumberDocs;
	private double numberDocsInClass;
	private double numberDocsOutClass;
	private int totalUniqueTerms;

	public TermStats() {

	}

	public TermStats(BagOfWords b, CategoryMatrix c, int i) 
	{
		this.countTerms(c, b, i);
	}

	public void countTerms(CategoryMatrix c, BagOfWords b, int cat)
	{
		
		
		catName = c.catName(cat);
		catID = cat + 1;
		
		
		
		// check that the docCounts are the same
		if(c.docCount() == b.docCount()) totalNumberDocs = b.docCount();
		else
		{
			System.err.println("number of documents in the categories "
					+ c.docCount() + " differ from the number or documents in the bow "
					+ b.docCount());
			System.exit(-1);
		}
		//extract a category vector
		int cvector[] = c.getCatVector(cat);

		// count the number of docs in and out of the cat
		numberDocsInClass = 0;
		numberDocsOutClass = 0;
		for (int i = 0; i < totalNumberDocs; i++)
		{
			if(cvector[i] == 1) numberDocsInClass++;
			else numberDocsOutClass++;
//			System.out.println("i/in/out/c" + i + "/" + numberDocsInClass
//			+ "/" + numberDocsOutClass + "/" +  cvector[i]);
		}

		if( (numberDocsInClass + numberDocsOutClass) != totalNumberDocs)
		{
			System.err.println("Document Counts do not add up: "
					+ numberDocsInClass + " in class and " + numberDocsOutClass
					+ " out of class w/ " + totalNumberDocs + " total");
			System.exit(-1);
		}

		System.out.println("Document Counts: "
				+ numberDocsInClass + " in class and " + numberDocsOutClass
				+ " out of class w/ " + totalNumberDocs + " total");
		
		double pInClass = numberDocsInClass/totalNumberDocs;
		double pOutClass = numberDocsOutClass/totalNumberDocs;
		
		System.out.println("the class prob: " + pInClass + " + " + pOutClass);

		// get a copy of the bow
		int bow[][] = b.getBow();
		int totalBowTerms = bow.length;


		// get the term ID hashmap
		TreeMap<Integer, String> IDs = b.getTermID();

		totalUniqueTerms = IDs.size();
		totalTermOccur = new double[totalUniqueTerms];
		termOccurInClass = new double[totalUniqueTerms];
		termOccurOutClass = new double[totalUniqueTerms];
		termNotOccurInClass = new double[totalUniqueTerms];
		termNotOccurOutClass = new double[totalUniqueTerms];
		freqTermTotal = new double[totalUniqueTerms];
		freqTermInClass = new double[totalUniqueTerms];
		freqTermOutClass = new double[totalUniqueTerms];
		mult = new double[totalUniqueTerms];
		IG = new double[totalUniqueTerms];
		norm = new double[totalUniqueTerms];
		
		double invLog2 = 1/Math.log(2);
		double invN = 1/totalNumberDocs;
		
		for(int i = 0; i < totalUniqueTerms; i++)
		{
			//initialize
			totalTermOccur[i] = 0;
			termOccurInClass[i] = 0;
			termOccurOutClass[i] = 0;
			termNotOccurInClass[i] = 0;
			termNotOccurOutClass[i] = 0;
			freqTermTotal[i] = 0;
			freqTermInClass[i] = 0;
			freqTermOutClass[i] = 0;
			mult[i] = 1;
			norm[i] = 0; // max term count in doc
			
			int bowTerm = 0;
			int currentDoc = 0; // note that the first index in the cat matrix
			// is zero but for the bow  and terms it is 1.
			while(bowTerm < totalBowTerms)
			{
				double freq = 0;
				double termCount;
				int currentCat = cvector[currentDoc];
				while((currentDoc + 1) == bow[bowTerm][0])
				{
					if(bow[bowTerm][1] == (i+1))
					{
						totalTermOccur[i]++;
						freq = (double)bow[bowTerm][2];
						if(freq > norm[i]) norm[i] = freq;
					}
					bowTerm++;
					if(bowTerm >= totalBowTerms) break;// in case the last doc does not have the term
				}

				// so now we have the category and if it occurs (freq > 0).
				if(currentCat == 1)
				{
					if(freq > 0)
					{
						termOccurInClass[i]++;
						freqTermInClass[i] += freq;
						freqTermTotal[i] += freq;
//						System.out.println(freq + " - ");
					}
					else
					{
						termNotOccurInClass[i]++;
					}
				}
				else
				{
					if(freq > 0)
					{
						termOccurOutClass[i]++;
						freqTermOutClass[i] += freq;
						freqTermTotal[i] += freq;
					}
					else
					{
						termNotOccurOutClass[i]++;
					}
				}
				currentDoc++;
			}
			double pTermOccur = totalTermOccur[i]*invN;
//			double pTermNotOccure = 1 - pTermOccur;
			double pTermNotOccure = (totalNumberDocs - totalTermOccur[i])*invN;// - pTermOccur;
			
			
			//System.out.printf("invN: %g\tinvLog2: %g\tpI: %g\tpT: %g\n", invN, invLog2, pInClass, pTermOccur);
			IG[i] = 0.0;
			double  tmp[] = new double[4];
//			// doc 1 class 1
			tmp[0] = termOccurInClass[i]*invN*invLog2
				*( Math.log(termOccurInClass[i]*invN/(pTermOccur*pInClass)));
			if(Double.isNaN(tmp[0])) tmp[0] = 0;
			IG[i] += tmp[0];
//			System.out.printf("d1c1: %g\t",tmp);

//			// doc 0 class 1
			tmp[1] = (termNotOccurInClass[i])*invN*invLog2
				*( Math.log((termNotOccurInClass[i])*invN/(pTermNotOccure*pInClass)));
			if(Double.isNaN(tmp[1])) tmp[1] = 0;
			IG[i] += tmp[1];
//			System.out.printf("d0c1: %g\t",tmp);
//			
			// doc 1 class 0
			tmp[2] = termOccurOutClass[i]*invN*invLog2
				*( Math.log(termOccurOutClass[i]*invN/(pTermOccur*pOutClass)));
			if(Double.isNaN(tmp[2])) tmp[2] = 0;
			IG[i] += tmp[2];
//			System.out.printf("d1c0: %g\t",tmp);
			
			
//			// doc 0 class 0
			tmp[3] = (termNotOccurOutClass[i])*invN*invLog2
				*( Math.log((termNotOccurOutClass[i])*invN/(pTermNotOccure*pOutClass)));
			if(Double.isNaN(tmp[3])) tmp[3] = 0;
			IG[i] += tmp[3];
//			System.out.printf("d0c0: %g\tIG: %g\n",tmp, IG[i]);
//			if(IG[i] < 0) System.out.printf("%d\t%d\tIG: %g - %g %g %g %g\n",cat+1, i+1, IG[i], tmp[0], tmp[1], tmp[2], tmp[3]);
//			
//			String output = "term: " + IDs.get(i+1) + " ID: " + (i+1);
//			output += " ave: " + freqTermTotal[i] + " ave_in_class: ";
//			output += freqTermInClass[i] + " ave_out_class: ";
//			output += freqTermOutClass[i] + " occure_in_class: ";
//			output += termOccurInClass[i] + " occure_out_class: ";
//			output += termOccurOutClass[i] + " total_occur: ";
//			output += totalTermOccur[i] + " IG: " + IG[i];
//			System.out.println(output);
			if(i%1000 == 0)
				System.out.println("Completed " + i + " of " + totalUniqueTerms);
		}
		
		
		



	}

	public void printTermStatsMatlab(String fileName) throws IOException
	{
		BufferedWriter ts = new BufferedWriter(new FileWriter(fileName));
		ts.write(String.format("totalDocs = %g;\ninClass = %g;\noutClass = %g;\ntotalTerms = %d;",
				totalNumberDocs, numberDocsInClass, numberDocsOutClass, totalUniqueTerms));
		ts.write("counts = [");
		for(int i = 0; i < totalUniqueTerms; i++)
			ts.write(String.format("%g,%g,%g,%g,%g,%g,%g,%g,%1.16E\n",
					freqTermTotal[i], freqTermInClass[i],
					freqTermOutClass[i], totalTermOccur[i],
					termOccurInClass[i], termNotOccurInClass[i],
					termOccurOutClass[i], termNotOccurOutClass[i],
					IG[i]));
		ts.write("];");
		ts.close();
		
	}
	
	public String getTermStatsString()
	{
		StringBuffer ts = new StringBuffer();
		ts.append(String.format("category_number,%d,totalDocs,%g,inClass,%g,outClass,%g,totalTerms,%d,catname,%s\n",
				catID, totalNumberDocs, numberDocsInClass, numberDocsOutClass, totalUniqueTerms, catName));
		ts.append(this.getHeader() + "\n");
		for(int i = 0; i < totalUniqueTerms; i++)
			ts.append(String.format("%g,%g,%g,%g,%g,%g,%g,%g,%g,%1.16E\n",
					freqTermTotal[i], freqTermInClass[i],
					freqTermOutClass[i], totalTermOccur[i],
					termOccurInClass[i], termNotOccurInClass[i],
					termOccurOutClass[i], termNotOccurOutClass[i],
					norm[i], IG[i]));
//		ts.append("\n");
		return ts.toString();
		
	}
	
	public double[] getTermStat(int i)
	{
		double termStat[] = new double[11];
		
		termStat[0] = freqTermTotal[i];
		termStat[1] = freqTermInClass[i];
		termStat[2] = freqTermOutClass[i];
		termStat[3] = totalTermOccur[i];
		termStat[4] = termOccurInClass[i];
		termStat[5] = termNotOccurInClass[i];
		termStat[6] = termOccurOutClass[i];
		termStat[7] = termNotOccurOutClass[i];
		termStat[8] = mult[i];
		termStat[9] = norm[i];
		termStat[10] = IG[i];
		
		return termStat;
		
	}
	
	
	
	public String getHeader()
	{
		String a = "term_freq,";
		a += catName + "_freq_inClass,";
		a += catName + "_freq_outClass,";
		a += "term_occur,";
		a += catName + "_occur_inClass,";
		a += catName + "_not_occur_inClass,";
		a += catName + "_occur_outClass,";
		a += catName + "_not_occur_outClass,";
		a += catName + "_term_multiplier,";
		a += catName + "_normalization,";
		a += catName + "_term_info_gain";
		
		return a;
	}
	
	public String getTermRank()
	{
		return Arrays.toString(IG);
	}
	
	public String getTermNorm()
	{
		return Arrays.toString(norm);
	}
	
	public double termNorm(int i)
	{
		if(i > norm.length)
		{
			System.err.print("norm index is out of range, ");
			System.err.println("Max = " + (norm.length - 1) +
					" current = " + i);
		}
		return norm[i - 1];
	}
	
	public void loadTermNorm(String normString)
	{
		normString = normString.replaceAll("\\s+|\\[|\\]", "");
		//System.out.println(normString);
		String[] d = normString.split(",");
		norm = new double[d.length];
		for(int i = 0; i < d.length; i++)
			norm[i] = Double.valueOf(d[i]);
	}
	
	public int[] reduceTerms(int count)
	{
		ArrayList<Integer> goodIG = new ArrayList<Integer>();
		int [] reducedID;
		if (count > IG.length - 1)
		{
			//System.out.println("count larger than IG.length: c/i" + count + "/" + IG.length);
			//include all nonzero IG
			for(int i = 0; i < IG.length; i++)
				if(IG[i] > 0) goodIG.add(i+1);
				
		}
		else
		{
			//System.out.println("count smaller than IG.length: c/i" + count + "/" + IG.length);

			double[] newIG = IG.clone();
			Arrays.sort(newIG);
			
			double minRank = newIG[newIG.length - 1 - count];
			for(int i = 0; i < IG.length; i++)
				if(IG[i] > minRank) goodIG.add(i+1); // the index of the data vs the IG array
		}
		reducedID = new int[goodIG.size()];
		Iterator<Integer> it = goodIG.iterator();
		int i = 0;
		while(it.hasNext() &&  i < goodIG.size())
		{
			reducedID[i] = it.next().intValue();
			i++;
		}
		
		
		
		// this is sorted in ascending order.
		// try with maps
//		HashMap<Double, Integer> igMap = new HashMap<Double, Integer>(IG.length);
//		for(int i = 1; i <IG.length; i++)
//		{
//			igMap.put(IG[i-1], i);
//		}
//		TreeMap<Double, Integer> igTree = new TreeMap<Double, Integer>(igMap);
//		
//		int j = 0;
//		Set keys = igTree.keySet();
		return reducedID;
	}
	
	public void loadTermRank(String igString)
	{
		igString = igString.replaceAll("\\s+|\\[|\\]", "");
		//System.out.println(igString);
		String[] d = igString.split(",");
		IG = new double[d.length];
		for(int i = 0; i < d.length; i++)
			IG[i] = Double.valueOf(d[i]);
	}

}
